@@ -2,8 +2,8 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: log.c,v 1.116 2006/03/13 00:54:21 nickm Exp $ */
-const char log_c_id[] = "$Id: log.c,v 1.116 2006/03/13 00:54:21 nickm Exp $";
+/* $Id: log.c,v 1.116.2.1 2006/06/09 06:54:02 arma Exp $ */
+const char log_c_id[] = "$Id: log.c,v 1.116.2.1 2006/06/09 06:54:02 arma Exp $";
 
 /**
  * \file log.c
@@ -339,8 +339,10 @@ delete_log(logfile_t *victim)
     logfiles = victim->next;
   else {
     for (tmpl = logfiles; tmpl && tmpl->next != victim; tmpl=tmpl->next) ;
-    tor_assert(tmpl);
-    tor_assert(tmpl->next == victim);
+//    tor_assert(tmpl);
+//    tor_assert(tmpl->next == victim);
+    if (!tmpl)
+      return;
     tmpl->next = victim->next;
   }
   tor_free(victim->filename);
