@@ -1,11 +1,11 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2006 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: tortls.h,v 1.33 2006/02/09 05:46:48 arma Exp $ */
+/* $Id: tortls.h,v 1.33.2.1 2006/06/06 09:04:50 arma Exp $ */
 
 #ifndef _TORTLS_H
 #define _TORTLS_H
-#define TORTLS_H_ID "$Id: tortls.h,v 1.33 2006/02/09 05:46:48 arma Exp $"
+#define TORTLS_H_ID "$Id: tortls.h,v 1.33.2.1 2006/06/06 09:04:50 arma Exp $"
 
 /**
  * \file tortls.h
@@ -32,7 +32,8 @@ tor_tls_t *tor_tls_new(int sock, int is_server, int use_no_cert);
 int tor_tls_is_server(tor_tls_t *tls);
 void tor_tls_free(tor_tls_t *tls);
 int tor_tls_peer_has_cert(tor_tls_t *tls);
-int tor_tls_get_peer_cert_nickname(tor_tls_t *tls, char *buf, size_t buflen);
+int tor_tls_get_peer_cert_nickname(int severity, tor_tls_t *tls,
+                                   char *buf, size_t buflen);
 int tor_tls_verify(int severity, tor_tls_t *tls, crypto_pk_env_t **identity);
 int tor_tls_check_lifetime(tor_tls_t *tls, int tolerance);
 int tor_tls_read(tor_tls_t *tls, char *cp, size_t len);
