@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.230.2.1 2006/04/18 03:05:34 nickm Exp $ */
+/* $Id: circuitbuild.c,v 1.230.2.6 2006/06/10 00:32:31 arma Exp $ */
 const char circuitbuild_c_id[] =
-  "$Id: circuitbuild.c,v 1.230.2.1 2006/04/18 03:05:34 nickm Exp $";
+  "$Id: circuitbuild.c,v 1.230.2.6 2006/06/10 00:32:31 arma Exp $";
 
 /**
  * \file circuitbuild.c
@@ -404,6 +404,7 @@ void
 circuit_n_conn_done(connection_t *or_conn, int status)
 {
   extern smartlist_t *circuits_pending_or_conns;
+  smartlist_t *changed_circs;
 
   log_debug(LD_CIRC,"or_conn to %s, status=%d",
             or_conn->nickname ? or_conn->nickname : "NULL", status);
@@ -411,6 +412,8 @@ circuit_n_conn_done(connection_t *or_conn, int status)
   if (!circuits_pending_or_conns)
     return;
 
+  changed_circs = smartlist_create();
+
   SMARTLIST_FOREACH(circuits_pending_or_conns, circuit_t *, circ,
   {
     if (circ->marked_for_close)
@@ -418,12 +421,18 @@ circuit_n_conn_done(connection_t *or_conn, int status)
     tor_assert(circ->state == CIRCUIT_STATE_OR_WAIT);
     if (!circ->n_conn &&
         circ->n_addr == or_conn->addr &&
-        circ->n_port == or_conn->port &&
-        !memcmp(or_conn->identity_digest, circ->n_conn_id_digest,
-                DIGEST_LEN)) {
+        circ->n_port == or_conn->port) {
+      if (memcmp(or_conn->identity_digest, circ->n_conn_id_digest,
+                 DIGEST_LEN)) {
+        log_fn(LOG_PROTOCOL_WARN, LD_CIRC,
+               "Pending circuit to %s:%d is intended for different digest!",
+               or_conn->address, or_conn->port);
+        circuit_mark_for_close(circ, END_CIRC_REASON_OR_IDENTITY);
+        continue;
+      }
       if (!status) { /* or_conn failed; close circ */
         log_info(LD_CIRC,"or_conn failed. Closing circ.");
-        circuit_mark_for_close(circ, END_CIRC_REASON_OR_IDENTITY);
+        circuit_mark_for_close(circ, END_CIRC_REASON_OR_CONN_CLOSED);
         continue;
       }
       log_debug(LD_CIRC,
@@ -449,10 +458,18 @@ circuit_n_conn_done(connection_t *or_conn, int status)
           continue;
         }
         tor_free(circ->onionskin);
-        circuit_set_state(circ, CIRCUIT_STATE_OPEN);
+        /* We don't want to change circ's state here, since the act
+         * of doing that modifies the circuits_pending_or_conns list
+         * that we're looping through right now. So collect a list of
+         * circs to change their state when we're done. */
+        smartlist_add(changed_circs, circ);
       }
     }
   });
+
+  SMARTLIST_FOREACH(changed_circs, circuit_t *, circ,
+    circuit_set_state(circ, CIRCUIT_STATE_OPEN));
+  smartlist_free(changed_circs);
 }
 
 /** Find a new circid that isn't currently in use on the circ->n_conn
@@ -1484,7 +1501,8 @@ choose_good_middle_server(uint8_t purpose,
     }
   }
   choice = router_choose_random_node(
-           NULL, get_options()->ExcludeNodes, excluded,
+           purpose == CIRCUIT_PURPOSE_TESTING ? get_options()->TestVia : NULL,
+           get_options()->ExcludeNodes, excluded,
            state->need_uptime, state->need_capacity, 0,
            get_options()->_AllowInvalid & ALLOW_INVALID_MIDDLE, 0);
   smartlist_free(excluded);
@@ -1531,6 +1549,14 @@ choose_good_entry_server(uint8_t purpose, cpath_build_state_t *state)
         smartlist_add(excluded, r);
     }
   }
+  /* and exclude current entry guards, if applicable */
+  if (options->UseEntryGuards && entry_guards) {
+    SMARTLIST_FOREACH(entry_guards, entry_guard_t *, entry,
+      {
+        if ((r = router_get_by_digest(entry->identity)))
+          smartlist_add(excluded, r);
+      });
+  }
   // XXX we should exclude busy exit nodes here, too,
   // but only if there are enough other nodes available.
   choice = router_choose_random_node(
@@ -2003,6 +2029,8 @@ entry_guard_set_status(const char *digest, int succeeded)
             SMARTLIST_FOREACH(entry_guards, entry_guard_t *, e,
               {
                 routerinfo_t *r;
+                if (e == entry)
+                  break;
                 if (e->made_contact) {
                   e->down_since = 0;
                   r = entry_is_live(e, 0, 1);
@@ -2011,8 +2039,6 @@ entry_guard_set_status(const char *digest, int succeeded)
                     r->is_running = 1;
                   }
                 }
-                if (e == entry)
-                  break;
               });
             log_info(LD_CIRC,
                      "Connected to new entry guard '%s'. Marking earlier "
