@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_or.c,v 1.216 2006/03/18 01:24:04 arma Exp $ */
+/* $Id: connection_or.c,v 1.216.2.3 2006/06/07 22:11:59 arma Exp $ */
 const char connection_or_c_id[] =
-  "$Id: connection_or.c,v 1.216 2006/03/18 01:24:04 arma Exp $";
+  "$Id: connection_or.c,v 1.216.2.3 2006/06/07 22:11:59 arma Exp $";
 
 /**
  * \file connection_or.c
@@ -581,11 +581,13 @@ connection_or_check_valid_handshake(connection_t *conn, char *digest_rcvd)
 
   check_no_tls_errors();
   if (! tor_tls_peer_has_cert(conn->tls)) {
-    log_info(LD_PROTOCOL,"Peer didn't send a cert! Closing.");
+    log_info(LD_PROTOCOL,"Peer (%s:%d) didn't send a cert! Closing.",
+             conn->address, conn->port);
     return -1;
   }
   check_no_tls_errors();
-  if (tor_tls_get_peer_cert_nickname(conn->tls, nickname, sizeof(nickname))) {
+  if (tor_tls_get_peer_cert_nickname(severity, conn->tls, nickname,
+                                     sizeof(nickname))) {
     log_fn(severity,LD_PROTOCOL,"Other side (%s:%d) has a cert without a "
            "valid nickname. Closing.",
            conn->address, conn->port);
@@ -689,6 +691,18 @@ connection_tls_finish_handshake(connection_t *conn)
 #endif
     connection_or_init_conn_from_address(conn,conn->addr,conn->port,
                                          digest_rcvd);
+    /* Annotate that we received a TLS connection.
+     * (Todo: only actually consider ourselves reachable if there
+     * exists a testing circuit using conn.)
+     *
+     * We already consider ourselves reachable if we can ever process
+     * a create cell -- see onionskin_answer() in circuitbuild.c.
+     *
+     * The reason this bandaid is here is because there's a bug in
+     * Tor 0.1.1.x where middle hops don't always send their create
+     * cell; so some servers rarely find themselves reachable. */
+//    if (!is_local_IP(conn->addr))
+//      router_orport_found_reachable();
   }
 
   if (!server_mode(get_options())) { /* If I'm an OP... */
