@@ -1,9 +1,9 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: cpuworker.c,v 1.98 2006/02/13 09:02:35 arma Exp $ */
+/* $Id: cpuworker.c,v 1.101.2.1 2006/04/25 07:31:16 arma Exp $ */
 const char cpuworker_c_id[] =
-  "$Id: cpuworker.c,v 1.98 2006/02/13 09:02:35 arma Exp $";
+  "$Id: cpuworker.c,v 1.101.2.1 2006/04/25 07:31:16 arma Exp $";
 
 /**
  * \file cpuworker.c
@@ -23,8 +23,6 @@ const char cpuworker_c_id[] =
 
 /** The tag specifies which circuit this onionskin was from. */
 #define TAG_LEN 8
-/** How many bytes are sent from tor to the cpuworker? */
-#define LEN_ONION_QUESTION (1+TAG_LEN+ONIONSKIN_CHALLENGE_LEN)
 /** How many bytes are sent from the cpuworker back to tor? */
 #define LEN_ONION_RESPONSE \
   (1+TAG_LEN+ONIONSKIN_REPLY_LEN+CPATH_KEY_MATERIAL_LEN)
@@ -261,7 +259,7 @@ cpuworker_main(void *data)
                  "(either rotated keys or died).");
       } else {
         log_info(LD_OR,
-                 "CPU worker editing because of error on connection to Tor "
+                 "CPU worker exiting because of error on connection to Tor "
                  "process.");
         log_info(LD_OR,"(Error on %d was %s)",
                  fd, tor_socket_strerror(tor_socket_errno(fd)));
@@ -398,12 +396,17 @@ process_pending_task(connection_t *cpuworker)
     log_warn(LD_OR,"assign_to_cpuworker failed. Ignoring.");
 }
 
-#define CPUWORKER_BUSY_TIMEOUT 3600 /* seconds */
+/** How long should we let a cpuworker stay busy before we give
+ * up on it and decide that we have a bug or infinite loop?
+ * This value is high because some servers with low memory/cpu
+ * sometimes spend an hour or more swapping, and Tor starves. */
+#define CPUWORKER_BUSY_TIMEOUT (60*60*12)
 
-/** We have a bug that I can't find. Sometimes, very rarely, cpuworkers
- * get stuck in the 'busy' state, even though the cpuworker process
- * thinks of itself as idle. I don't know why. But here's a workaround
- * to kill any cpuworker that's been busy for more than 3600 seconds. */
+/** We have a bug that I can't find. Sometimes, very rarely, cpuworkers get
+ * stuck in the 'busy' state, even though the cpuworker process thinks of
+ * itself as idle. I don't know why. But here's a workaround to kill any
+ * cpuworker that's been busy for more than CPUWORKER_BUSY_TIMEOUT.
+ */
 static void
 cull_wedged_cpuworkers(void)
 {
