@@ -1,9 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.370 2006/04/10 01:35:56 nickm Exp $ */
+/* $Id: directory.c,v 1.370.2.1 2006/04/18 03:05:34 nickm Exp $ */
 const char directory_c_id[] =
-  "$Id: directory.c,v 1.370 2006/04/10 01:35:56 nickm Exp $";
+  "$Id: directory.c,v 1.370.2.1 2006/04/18 03:05:34 nickm Exp $";
 
 #include "or.h"
 
@@ -534,8 +534,8 @@ directory_send_command(connection_t *conn, const char *platform,
 
       httpcommand = "GET";
       /* Request the most recent versioned descriptor. */
-      // XXXX011
-      //tor_snprintf(url, sizeof(url), "/tor/rendezvous1/%s", resource);
+      // (XXXX We were going to switch this to fetch rendezvous1 descriptors,
+      // but that never got testing, and it wasn't a good design.)
       len = strlen(resource)+32;
       url = tor_malloc(len);
       tor_snprintf(url, len, "/tor/rendezvous/%s", resource);
