@@ -1,8 +1,8 @@
 /* Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: hibernate.c,v 1.70 2006/03/13 01:06:55 nickm Exp $ */
+/* $Id: hibernate.c,v 1.70.2.1 2006/04/25 07:31:16 arma Exp $ */
 const char hibernate_c_id[] =
-  "$Id: hibernate.c,v 1.70 2006/03/13 01:06:55 nickm Exp $";
+  "$Id: hibernate.c,v 1.70.2.1 2006/04/25 07:31:16 arma Exp $";
 
 /**
  * \file hibernate.c
@@ -862,9 +862,10 @@ consider_hibernation(time_t now)
                  "Bandwidth soft limit reached; commencing hibernation.");
       hibernate_begin(HIBERNATE_STATE_LOWBANDWIDTH, now);
     } else if (accounting_enabled && now < interval_wakeup_time) {
-      format_iso_time(buf,interval_wakeup_time);
+      format_local_iso_time(buf,interval_wakeup_time);
       log_notice(LD_ACCT,
-                 "Commencing hibernation. We will wake up at %s GMT", buf);
+                 "Commencing hibernation. We will wake up at %s local time.",
+                 buf);
       hibernate_go_dormant(now);
     }
   }
